/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Epipolar.cpp
 *
 *  Created on: 2010/11/15
 *      Author: HARK Development Team
 */

#include <AngleConstants.hpp>
#include <DimensionPromise/dimpro.hpp>
#include <Epipolar.hpp>
#include <cmath>

using namespace NSDimensionPromise;

namespace NSHarkBinaural
{

Length Epipolar::LRDifferenceSimple(Angle sourceDirection) const
{
    const Length r = this->headRadius;
    const Length len = this->sourceDistance;

    Length lenl, lenr;
    lenl = sqrt(r * r + len * len - 2.0 * r * len * sin( sourceDirection ));
    lenr = sqrt(r * r + len * len + 2.0 * r * len * sin( sourceDirection ));
    return ((lenl - lenr) * 4.0) / 3.0;
}

Length Epipolar::LRDifferenceRevised(Angle sourceDirection) const
{
    const Length r = this->headRadius;
    const Length len = this->sourceDistance;

    Angle th;
    Length ll, ll1(0.0), lr1(0.0);
    bool minus = sourceDirection.ToRadian() < 0;
    Angle theta = fabs( sourceDirection );

    th = acos( r / len );
    ll = sqrt( len * len - r * r );

    if ( theta < Angles::Zero )
    {
        // nothing to do
    }
    else if ( Angles::Zero <= theta && theta < Angles::AnglePi2 - th )
    {
        ll1 = ll + Angle(Angles::AnglePi2 - theta - th).ToRadian() * r;
        lr1 = ll + Angle(Angles::AnglePi2 + theta - th).ToRadian() * r;
    }
    else if ( Angles::AnglePi2 - th <= theta && theta < Angles::AnglePi2 )
    {
        ll1 = sqrt( len * len + r * r - 2.0 * r * len * sin( theta ) );
        lr1 = ll + ( Angles::AnglePi2 + theta - th ) * r;
    }
    else if ( Angles::AnglePi2 <= theta && theta < Angles::AnglePi2 + th )
    {
        ll1 = sqrt( len * len + r * r - 2.0 * r * len * sin( theta ) );
        lr1 = ll + ( 3.0 * Angles::AnglePi2 - theta - th ) * r;
    }
    else if ( Angles::AnglePi2 + th <= theta && theta < 3.0 * Angles::AnglePi2 - th)
    {
        ll1 = ll + ( -Angles::AnglePi2 + theta - th ) * r;
        lr1 = ll + ( 3.0 * Angles::AnglePi2 - theta - th ) * r;
    }
    else if ( 3.0 * Angles::AnglePi2 - th <= theta && theta < 3.0 * Angles::AnglePi2)
    {
        ll1 = ll + ( -Angles::AnglePi2 + theta - th ) * r;
        lr1 = sqrt( len * len + r * r + 2.0 * r * len * sin(theta) );
    }
    else if ( Angles::AnglePi2 <= theta && theta < 3.0 * Angles::AnglePi2 + th )
    {
        ll1 = ll + ( 5.0 * Angles::AnglePi2 - theta - th ) * r;
        lr1 = sqrt( len * len + r * r + 2.0 * r * len * sin(theta) );
    }
    else if ( 3.0 * Angles::AnglePi2 + th <= theta && theta <= 2.0 * Angles::AnglePi )
    {
        ll1 = ll + ( 5.0 * Angles::AnglePi2 - theta - th ) * r;
        lr1 = ll + ( -3.0 * Angles::AnglePi2 + theta - th ) * r;
    }

    return minus ? lr1 - ll1 : ll1 - lr1;

}

}
